from turkish_syllable import syllabify

print(syllabify("halkların astların testlerin bantlamak da gerekir kelimesi budur, ardından neler gelir bilemiyorum!", with_punctuation=False))
print(syllabify("aktivist", with_punctuation=True))
